#ifndef _INCLUDE_ANIMATION_H_
#define _INCLUDE_ANIMATION_H_

/* 
Jumping Jack by Stanislav Tihohod 
Copyright (c) 2002 by Stanislav Tihohod. 
*/

#include "ScrObject.h"
#include "Sprite.h"

struct TAnimFrame
{
	int m_FrameID;
	int dx,dy;
};

struct TAnimSoundFrame
{
	int m_FrameID;
	int dx,dy;
	int Freq;
	int Duration;
};

struct TSoundDescriptor
{
	int m_Freq;
	int m_Duration;
};

struct TAnimDescriptor
{
	TAnimFrame *m_pFrames;
	int m_NFrames;
	int m_Speed;
};

class CAnimation: public CScreenObject
{
private:

	static bool m_bPauseMode;
	
public:
	
	static bool setPauseMode(bool bMode=true);
	static bool getPauseMode(void) {return m_bPauseMode;}

private:

	int m_NSprites;
	CSprite **m_pSprite;
	
	TSoundDescriptor *m_pSound;
		
	CPos *m_pPos;
	int m_NFrames;
	int *m_pFrameSpriteIndex;
	
public:	
	int m_CurrentFrameIndex;
	int m_CurrentDraw;
	
private:
	int m_PlaySpeed;
	
	bool m_bShow;
	bool m_bCycled;
	
public:
	
	void create(TAnimFrame *pFrame, int NFrames, int x, int y,  int speed, bool bCycled, bool bShow=true);
	void create(TAnimSoundFrame *pFrame, int NFrames, int x, int y,  int speed, bool bCycled, bool bShow=true);	
	virtual void draw(bool InvalidateArea=true);
	virtual void show(bool bShow=true);	
	virtual void setPos(int x, int y);
	virtual CDrawSurface *setDrawSurface(CDrawSurface *pNewSurface);	
	CSprite *getFrame(int index);
	
public:

	void reset(void);	//      .  .
	void restart(void); //      .  .
	bool isFinished(void);
	
public:

	CAnimation();
	virtual ~CAnimation();
	
private:

	int getUniqeFrameIDNumber(TAnimFrame *pFrameID, int NFrames);
	int getUniqeFrameIDNumber(TAnimSoundFrame *pFrameID, int NFrames);	
};

#endif